#include "triccagantry.h"
#include "gantrycontroller.h"
#include "wellplateprotocols.h"

void gantry_controller_setup() {
    gantry_controller.loop_cond = 1;
    gantry_controller.loop = gantry_control;
	gantry_controller.move_dist_mm = 9.0;
	gantry_controller.move_axis = X_AXIS_MACHINE;

    gantry_controller.move_buttons[0].btn_pressed = btn_pressed;
    gantry_controller.move_buttons[1].btn_pressed = btn_pressed;
    gantry_controller.move_buttons[2].btn_pressed = btn_pressed;
    gantry_controller.move_buttons[3].btn_pressed = btn_pressed;
    gantry_controller.move_buttons[4].btn_pressed = btn_pressed;
    gantry_controller.move_buttons[5].btn_pressed = btn_pressed;
    gantry_controller.move_buttons[6].btn_pressed = btn_pressed;

    gantry_controller.move_buttons[0].id = BTN8;
    gantry_controller.move_buttons[1].id = BTN9;
    gantry_controller.move_buttons[2].id = BTN10;
    gantry_controller.move_buttons[3].id = BTN11;
    gantry_controller.move_buttons[4].id = BTN12;
    gantry_controller.move_buttons[5].id = BTN13;
    gantry_controller.move_buttons[6].id = BTN14;

    gantry_controller.move_buttons[0].btn_func = inc_dist;
    gantry_controller.move_buttons[1].btn_func = cycle_dist;
    gantry_controller.move_buttons[2].btn_func = dec_dist;
    gantry_controller.move_buttons[3].btn_func = cycle_axis;
    gantry_controller.move_buttons[4].btn_func = break_out;
    gantry_controller.move_buttons[5].btn_func = travel_well_plate_btn;
    gantry_controller.move_buttons[6].btn_func = report_coor;
}

void report_coor() {
    struct gantry_coor coor = gantry_curr_coor;
    gantry_goto_coor_d(100, 300, 200);
    report_num(coor.x_pos);
    gantry_goto_coor_d(100, 300, 200);
    report_num(coor.y_pos);
    gantry_goto_coor_d(100, 300, 200);
    report_num(coor.z_pos);
    gantry_goto_coor(coor);
    btn_debounce();
}

void gantry_control() {
	while (gantry_controller.loop_cond) {
        for (int i = 0; i < 7; i++) {
            if (button_status(i+8)) {
                gantry_controller.move_buttons[i].btn_func();
            }
        }
	}
    gantry_controller.loop_cond = 1;
}

int btn_pressed(enum Buttons btnID) {
    return button_status(btnID);
}

void btn_debounce() {
    k_msleep(BTN_DEBOUNCE);
}

void cycle_dist() {
    // ... -> 9.0 (Start) -> 5.0 -> 1.0 -> 0.5 -> 0.1 (Restart) -> ... : Default: 10.0
	if (gantry_controller.move_dist_mm == 9.0) {
    	gantry_controller.move_dist_mm = 5.0;
	} else if (gantry_controller.move_dist_mm == 5.0) {
		 gantry_controller.move_dist_mm = 1.0;
    } else if (gantry_controller.move_dist_mm == 1.0) {
    	gantry_controller.move_dist_mm =  0.5;
	} else if (gantry_controller.move_dist_mm == 0.5) {
		gantry_controller.move_dist_mm = 0.1;
	} else if (gantry_controller.move_dist_mm == 0.1) {
        gantry_controller.move_dist_mm = 9.0;
    } else {
        gantry_controller.move_dist_mm = 9.0;
    }
    btn_debounce();
}

void cycle_axis() {
    // ... -> x_axis (Start) -> y_axis -> z_axis (Restart) -> ... : Default: x_axis
    switch (gantry_controller.move_axis) {
        case X_AXIS_MACHINE:
            gantry_controller.move_axis = Y_AXIS_MACHINE;
            break;
        case Y_AXIS_MACHINE:
            gantry_controller.move_axis = Z_AXIS_MACHINE;
            break;
        case Z_AXIS_MACHINE:
            gantry_controller.move_axis = X_AXIS_MACHINE;
            break;
        default:
            gantry_controller.move_axis = X_AXIS_MACHINE;
            break;
    }
    btn_debounce();
}

void inc_dist() {
    struct gantry_coor coor = gantry_curr_coor;
    switch (gantry_controller.move_axis) {
        case X_AXIS_MACHINE:
            coor.x_pos += gantry_controller.move_dist_mm;
            break;
        case Y_AXIS_MACHINE:
            coor.y_pos += gantry_controller.move_dist_mm;
            break;
        case Z_AXIS_MACHINE:
            coor.z_pos += gantry_controller.move_dist_mm;
            break;
        default:
            break;
    }
    gantry_goto_coor(coor);
    btn_debounce();
}

void dec_dist() {
    struct gantry_coor coor = gantry_curr_coor;
    switch (gantry_controller.move_axis) {
        case X_AXIS_MACHINE:
            coor.x_pos -= gantry_controller.move_dist_mm;
            break;
        case Y_AXIS_MACHINE:
            coor.y_pos -= gantry_controller.move_dist_mm;
            break;
        case Z_AXIS_MACHINE:
            coor.z_pos -= gantry_controller.move_dist_mm;
            break;
        default:
            break;
    }
    gantry_goto_coor(coor);
    btn_debounce();
}

void break_out() {
    gantry_controller.loop_cond = 0;
    btn_debounce();
}

void travel_well_plate_btn() {
    travel_well_plate(WELL_PLATES[0]);
}

void follow_func(struct gantry_coor (*funcp)(struct gantry_coor, double), struct gantry_coor start_pos, double range_min, double range_max, int res) {    
    struct gantry_coor coor;
    double func_input;
    double delta = range_max - range_min;
    if (res == 0)
        return;
    delta = delta / ((double) res);
    //Generate coordinates along function
    for (int i = 0; i < res; i++) {
        func_input = delta*i;
        func_input += range_min;
        coor = (*funcp)(start_pos, func_input);
        gantry_goto_coor(coor);
    }
}

void report_num(double num) {
    // Report the digits of pow 2 through -3
    double temp = num;
    struct gantry_coor coor = gantry_curr_coor;
    for (int i = 5; i >= 0; i--) {
        for (int j = 1; j < 11; j++) { // Iterate through all 10 digits + 1
            if (pow(10.0, i - 3)*j > temp) { // Check if the digit is larger than num, if larger then the digit is 1 less than the digit 
                 temp -= pow(10.0, i - 3)*(j - 1); // Remove largest digit
                 for (int k = 0; k < j; k++) { // Move the head to communicate found digit
                    coor.x_pos += 20;
                    gantry_goto_coor(coor);
                    k_msleep(500);
                 }
                 k_msleep(500);
                 for (int k = 0; k < j; k++) { // Move the head to communicate found digit
                    coor.x_pos -= 20;
                    gantry_goto_coor(coor);
                 }
                 break;
            }
        }
        coor.y_pos -= 20;
        gantry_goto_coor(coor);
        k_msleep(500);
    }
    coor.y_pos += 120.0;
    gantry_goto_coor(coor);
}

double vec3Mag(struct vec3 vec) {
    return sqrt(pow(vec.x_mag, 2.0) + pow(vec.y_mag, 2.0) + pow(vec.z_mag, 2.0));
}

double vec3Dot(struct vec3 vec1, struct vec3 vec2) {
    return (vec1.x_mag * vec2.x_mag) + (vec1.y_mag * vec2.y_mag) + (vec1.z_mag * vec2.z_mag);
}

struct vec3 vec3Cross(struct vec3 vec1, struct vec3 vec2) {
    struct vec3 newVec;
    newVec.x_mag = (vec1.y_mag * vec2.z_mag) - (vec1.z_mag * vec2.y_mag);
    newVec.y_mag = (vec1.z_mag * vec2.x_mag) - (vec1.x_mag * vec2.z_mag);
    newVec.z_mag = (vec1.x_mag * vec2.y_mag) - (vec1.y_mag * vec2.x_mag);
    return newVec;
}

double vec3AngleBtwn(struct vec3 vec1, struct vec3 vec2) {
    double denom = vec3Mag(vec1) * vec3Mag(vec2);
    return acos(vec3Dot(vec1, vec2) / denom);
}