#ifndef GANTRYCONTROLLER_H
#define GANTRYCONTROLLER_H

#include "triccaboard.h"
#include "triccagantry.h"

struct vec3 {
    double x_mag;
    double y_mag;
    double z_mag;
};

struct gantry_plane {
    struct vec3 x_vec;
    struct vec3 y_vec;
    struct vec3 z_vec;
} gantry_curr_plane;

struct gantry_button {
    int (*btn_pressed)(enum Buttons);
    int id;
    void (*btn_func)(void);
};
double BTN_DEBOUNCE = 200.0;

struct gantry_controller {
    void (*loop)(void);
    int loop_cond;
    struct gantry_button move_buttons[7];
    double move_dist_mm;
    enum gantry_axis move_axis;
} gantry_controller;

// Gantry Control Related Functions
void gantry_controller_setup();
void gantry_control();
void follow_func(struct gantry_coor (*funcp)(struct gantry_coor, double), struct gantry_coor start_pos, double range_min, double range_max, int res);
void report_coor();
void report_num(double num);
void cycle_dist();
void cycle_axis();
void dec_dist();
void inc_dist();
void break_out();
void btn_debounce();
int btn_pressed(enum Buttons btnID);
void travel_well_plate_btn();

double vec3Mag(struct vec3 vec);
double vec3Dot(struct vec3 vec1, struct vec3 vec2);
struct vec3 vec3Cross(struct vec3 vec1, struct vec3 vec2);
double vec3AngleBtwn(struct vec3 vec1, struct vec3 vec2);

#endif  // GANTRYCONTROLLER_H