#include "servo.h"
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>

//servo function mapping
int angmin = 0;
int angmax = 180;
int pulmin = 750;
int pulout = 2250;


void timer_handler(struct k_timer *timer_id)
{
    k_timer_stop(&my_timer); // Stop the timer
}

//mathematically map an input angle to an output pulse value
long angleToPulseMap(long x, long angmin, long angmax, long pulmin, long pulout)
{
  return ((x - angmin)*(pulout-pulmin)/(angmax-angmin)) + pulmin;
}

//send pulses to a servo to hold it at a given angle
void servoFunction(int servoID, long angle, int pulseNum)
{
    //limit angle to upper bound
    if (angle > 180)
    {
        angle = 180;
    }

    //limit angle to lower bound
    else if (angle < 0)
    {
        angle = 0;
    }

    long pulseLength = angleToPulseMap(angle, angmin, angmax, pulmin, pulout);

    //send a given number of pulses to hold the servo in place
    for (int i = 0; i < pulseNum; i++)
    {
        if (servoID == 2)
        {
            gpio_pin_toggle_dt(&serv2_dev);  //toggle on
            k_usleep(pulseLength);           //time on
            gpio_pin_toggle_dt(&serv2_dev);  //toggle off
            k_usleep(2250-pulseLength);      //time off
        }
        else if (servoID == 3)
        {
            gpio_pin_toggle_dt(&serv3_dev);  //toggle on
            k_usleep(pulseLength);           //time on
            gpio_pin_toggle_dt(&serv3_dev);  //toggle off
            k_usleep(2250-pulseLength);      //time off
        }
    }
}


void homeServo(int servoID, int homeAngle, int pulseNum)
{
    k_timer_start(&my_timer, K_MSEC(800), K_NO_WAIT);

    while (1)
    {
        servoFunction(servoID, homeAngle, pulseNum);

        if (k_timer_status_get(&my_timer) > 0)
        {
            break;
        }
        
        k_sleep(K_MSEC(50));
    }
}


void executeAngleSequence(int angles[], int num_angles, int duration_ms, int pulseNum, int servoID)
{
    for (int i = 0; i < num_angles; i++)
    {
        k_timer_start(&my_timer, K_MSEC(duration_ms), K_NO_WAIT);

        while (1)
        {
            servoFunction(servoID, angles[i], pulseNum);

             k_sleep(K_MSEC(50));

            if (k_timer_status_get(&my_timer) > 0)
            {
                break;
            }

             k_sleep(K_MSEC(50));
        }

        k_sleep(K_MSEC(500));  // Adjust the delay duration as needed (when changing angles)
    }
}