#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <math.h>
#include "triccaboard.c"
#include "triccagantry.c"
#include "wellplateprotocols.c"
#include "gantrycontroller.c"

#define SLEEP_TIME_MS	1000
#define M_PI    3.14159265358979323846

//servo function mapping (set as constants if needed) 
long angmin = 0;
long angmax = 180;
long pulmin = 750;
long pulout = 2250;
int home_angle = 15; // modify if needed

long angleToPulseMap(long x, long angmin, long angmax, long pulmin, long pulout);

void servoFunction(int servoID, long angle, int pulseNum);

//mathematically map an input angle to an output pulse value
long angleToPulseMap(long x, long angmin, long angmax, long pulmin, long pulout)
{
  return ((x - angmin)*(pulout-pulmin)/(angmax-angmin)) + pulmin;
}

//send pulses to a servo to hold it at a given angle
void servoFunction(int servoID, long angle, int pulseNum)
{
    //limit angle to upper bound
    if (angle > 180)
    {
        angle = 180;
    }
    //limit angle to lower bound
    else if (angle < 0)
    {
        angle = 0;
    }
    
    //map the angle to a given pulse length
    long pulseLength = angleToPulseMap(angle, 0, 180, 750, 2250);

    
    int i;
    //send a given number of pulses to hold the servo in place
    for (i = 0; i < pulseNum; i++)
    {
        
        if (servoID == 2)
        {
            gpio_pin_toggle_dt(&serv2_dev);  //toggle on
            k_usleep(pulseLength);           //time on
            gpio_pin_toggle_dt(&serv2_dev);  //toggle off
            k_usleep(2250-pulseLength);      //time off
        }
        else if (servoID == 3)
        {
            gpio_pin_toggle_dt(&serv3_dev);  //toggle on
            k_usleep(pulseLength);           //time on
            gpio_pin_toggle_dt(&serv3_dev);  //toggle off
            k_usleep(2250-pulseLength);      //time off
        }
    }
}

int main(void)
{
    gpio_setup();


    while(1){
        servoFunction(2, 90, 2);
    }
}
