#ifndef TRICCAGANTRY_H
#define TRICCAGANTRY_H
#include "triccaboard.h"

#define NUM_MOTOR_Z0 8
#define NUM_MOTOR_Z1 9
#define NUM_MOTOR_Z2 10
#define NUM_MOTOR_Z3 11
#define NUM_MOTOR_XY0 12
#define NUM_MOTOR_XY1 13

#define MAX_Y_UM 350000
#define MAX_Z_UM 350000
#define MAX_X_UM 350000

double Z_STEP_DELAY_US = 200.0;
double XY_STEP_DELAY_US = 200.0;

enum gantry_axis {
    X_AXIS_MACHINE,
    Y_AXIS_MACHINE,
    Z_AXIS_MACHINE
    // X_AXIS_PLANE, 
    // Y_AXIS_PLANE, 
    // Z_AXIS_PLANE
};

struct gantry_move_unit {
    double dist_um;
    enum gantry_axis axis;
    int dir;
};

struct gantry_coor {
    double x_pos;
    double y_pos;
    double z_pos;
};

struct gantry_coor gantry_curr_coor;

enum Buttons Z0_LIMIT = BTN0;
enum Buttons Z1_LIMIT = BTN1;
enum Buttons Z2_LIMIT = BTN2;
enum Buttons Z3_LIMIT = BTN3;
enum Buttons X_LIMIT = BTN4;
enum Buttons Y_LIMIT = BTN5;

void gantry_setup();
void stepZ(double steps, int dir, double delay_us);
void stepZ0(double steps, int dir, double delay_us);
void stepZ1(double steps, int dir, double delay_us);
void stepZ2(double steps, int dir, double delay_us);
void stepZ3(double steps, int dir, double delay_us);

void stepX(double steps, int dir, double delay_us);
void stepY(double steps, int dir, double delay_us);

void step(enum gantry_axis axis, double steps, int dir, double delay_us);

void gantry_home();
double gantry_home_x();
double gantry_home_y();
double gantry_home_z();
void gantry_home_return();
void gantry_move(struct gantry_move_unit move);
void gantry_move_exe(struct gantry_move_unit* move, int num_moves);
void gantry_goto_coor(struct gantry_coor coor);
void gantry_goto_coor_d(double x, double y, double z);
void gantry_set_delay_xy(double step_delay_us);
void gantry_set_delay_z(double step_delay_us);

#endif // TRICCAGANTRY_H