#include <math.h>
#include "wellplateprotocols.h"
#include "triccagantry.h"

#define M_PI		3.14159265358979323846

void travel_well_plate(struct plates plate) {
    gantry_goto_coor(plate.coor);
    for (int row = ROW_A; row <=ROW_H; row++) {
        for (int col = COL_1; col <= COL_12; col++) {
            gantry_home_z();
            // Go to each well
            travel_well_pos(plate, col, row);
            // Execute each well's function
            WELL_FUNCS[row*COL_TOTAL+col]();
        }
        gantry_home();
    }
    gantry_goto_coor(plate.coor);
    struct gantry_move_unit move;
    gantry_home_z();   
}

void travel_well_pos(struct plates plate, enum Well_Col col, enum Well_Row row) {
    struct gantry_coor var_pos = plate.coor;
    var_pos.x_pos += col * WELL_DIST_X;
    var_pos.y_pos += row * WELL_DIST_Y;
    gantry_goto_coor(var_pos);
}

void well_plate_setup() {
    WELL_PLATES[0].coor.x_pos = 17.824;
    WELL_PLATES[0].coor.y_pos = 347.100;
    WELL_PLATES[0].coor.z_pos = 119.109;
    for (int i = 0; i < 96; i++) {
        WELL_FUNCS[i] = dip_z_linear;
    }
}

void dip_z_linear() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    move.dir = 0;
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
}

void dip_z_linear_slow() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    move.dir = 0;
    double old_speed = Z_STEP_DELAY_US;
    gantry_set_delay_z(Z_STEP_DELAY_US * 1.5);
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
    gantry_set_delay_z(old_speed);
}

void dip_z_linear_slower() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    move.dir = 0;
    double old_speed = Z_STEP_DELAY_US;
    gantry_set_delay_z(Z_STEP_DELAY_US * 2.0);
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
    gantry_set_delay_z(old_speed);
}

void dip_z_linear_slowest() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    move.dir = 0;
    double old_speed = Z_STEP_DELAY_US;
    gantry_set_delay_z(Z_STEP_DELAY_US * 10.0);
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
    gantry_set_delay_z(old_speed);
}

void dip_z_linear_fast() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    move.dir = 0;
    double old_speed = Z_STEP_DELAY_US;
    gantry_set_delay_z(90);
    gantry_move(move);
    move.dir = 1;
    gantry_set_delay_z(old_speed);
    gantry_move(move);
}

void dip_z_double() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    move.dir = 0;
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
    move.dir = 0;
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
}

void dip_z_double_slow() {
    struct gantry_move_unit move;
    move.dist_um = 22500.0;
    move.axis = Z_AXIS_MACHINE;
    double old_speed = Z_STEP_DELAY_US;
    gantry_set_delay_z(Z_STEP_DELAY_US * 10.0);
    move.dir = 0;
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
    move.dir = 0;
    gantry_move(move);
    move.dir = 1;
    gantry_move(move);
    gantry_set_delay_z(old_speed);
}

void dip_z_ramp_slow_fast() {
    double old_speed = Z_AXIS_MACHINE;
    struct gantry_move_unit move;
    move.dist_um = 2250.0;
    move.axis = Z_AXIS_MACHINE;
    gantry_set_delay_z(Z_AXIS_MACHINE * 2.0);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.9);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.8);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.7);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.6);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.5);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.4);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.3);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.2);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.1);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
}

void dip_z_ramp_fast_slow() {
    double old_speed = Z_AXIS_MACHINE;
    struct gantry_move_unit move;
    move.dist_um = 2250.0;
    move.axis = Z_AXIS_MACHINE;
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.1);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.2);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.3);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.4);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.5);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.6);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.7);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.8);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 1.9);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
    gantry_set_delay_z(Z_AXIS_MACHINE * 2.0);
    gantry_move(move);
    gantry_set_delay_z(old_speed);
}

double map_range_linear(double in_min, double in_max, double o_min, double o_max, double in) {
    return (o_max - o_min) / (in_max - in_min) * (in - in_min) + o_min;
}

double map_range_sinusoidal(double in_min, double in_max, double o_min, double o_max, double in) {
    double pt1 = -(o_max - o_min) / 2;
    double pt2 = cos((M_PI * (in_min - in)) / (in_min - in_max));
    double pt3 = (o_max + o_min) / 2;
    return pt1*pt2+pt3;
}

struct gantry_coor travel_circle(struct gantry_coor start_pos, double theta) {
    struct gantry_coor new_pos;
    new_pos.x_pos = 10.0 * cos(theta);
    new_pos.y_pos = 10.0 * sin(theta);
    return new_pos;
}